import { Webview, ExtensionContext, WebviewOptions, Uri, ViewColumn, WebviewPanel, WebviewPanelOptions, WebviewViewProvider, WebviewView } from 'vscode';
import { AsyncStateful, AsyncState } from 'vscode-webview-shared';
export { AsyncState, AsyncStateful, ManualPromise, errorToString } from 'vscode-webview-shared';

declare abstract class VSCodeWebviewAPI<StateType extends object = Record<string, never>> implements AsyncStateful<StateType> {
    #private;
    readonly state: AsyncState<StateType>;
    constructor(webview: Webview);
    dispose(): void;
}

interface ViewOptions<API extends VSCodeWebviewAPI<any>> {
    path: string;
    index?: string;
    title: string;
    context: ExtensionContext;
    type?: string;
    options?: WebviewOptions;
    apiFactory?: (webview: Webview) => API;
}

interface ViewColumnWithFocus {
    viewColumn: ViewColumn;
    preserveFocus?: boolean;
}
type ShowOptions = ViewColumn | ViewColumnWithFocus;
type PanelOptions = WebviewPanelOptions & WebviewOptions;
interface Options<API extends VSCodeWebviewAPI<any>> extends ViewOptions<API> {
    showOptions?: ShowOptions;
    options?: PanelOptions;
}
declare class VSCodeWebview<API extends VSCodeWebviewAPI<any> = VSCodeWebviewAPI<any>> {
    protected readonly baseUri: Uri;
    private readonly title;
    readonly type: string;
    private readonly showOptions;
    protected readonly options: PanelOptions | undefined;
    private panel;
    protected readonly apiFactory: Options<API>['apiFactory'];
    protected api: API | undefined;
    private htmlPromise;
    private html;
    private readonly index;
    private apiPromise;
    constructor({ path, index, title, type, showOptions, context, apiFactory, options, }: Options<API>);
    protected buildHtml(baseSrc: Uri): Promise<string>;
    protected buildAPI(webview: Webview): void;
    private buildPanel;
    protected treatHTML(html: string, baseSrc: Uri): string;
    show(column?: ViewColumn): Promise<void>;
    getPanel(): WebviewPanel | undefined;
    getHTML(): string | undefined;
    getAPI(): API | undefined;
    getAPIAsync(): Promise<API>;
}

declare class VSCodeViewProvider<API extends VSCodeWebviewAPI<any> = VSCodeWebviewAPI<any>> extends VSCodeWebview<API> implements WebviewViewProvider {
    private view;
    constructor(options: ViewOptions<API>);
    resolveWebviewView(view: WebviewView): Promise<void>;
    show(): Promise<void>;
    getView(): WebviewView | undefined;
}

export { VSCodeViewProvider, VSCodeWebview, VSCodeWebviewAPI };
